@@hide_actionbar

!! Game Demo 

 The original hBasiX / libGDX A Simple Game 
 converted for hBasic v5.00+

 - v1.0 gdemo1 humpty Apr 2024
!!

gWID = 480 : gHGT = 800         % development size

gr.open "lightblue",0,-1
gr.screen rWID,rHGT             % real WxH
if rWID > rHGT then ori=0 else ori=1
if ori=0 then swap gWID,gHGT    % if landscape, change dev ori
gr.orientation ori              % force fix the ori

sW=rWID/gWID : sH=rHGT/gHGT     % get the ratios
scale = min (sW,sH)             % keep inside
gr.scale scale,scale            % scale to device

half = gWID/2
base = gHGT*0.6                 % stand here
!-------------------------------
! Player sprite os0

gr.bitmap.load b0,"h-stand.png"
gr.bitmap.scaf b0,3         % 3x enlarge inplace
gr.bitmap.size b0,pW,pH     % player size
                            % standing sprite
gr.sprite.draw os0,gWID/2-pW/2,base-pH,b0
                % load animations w,h,frames{,offset}
gr.anim.load a1,"h-anim.png",32,33,16
gr.anim.load a2,"h-anim.png",32,33,16,16 %..offset
gr.anim.scaf a1,3   % enlarge left animation
gr.anim.scaf a2,3   % enlarge right animation
gr.modify os0,"arate",16    % animation rate
pSpeed=250                  % player speed
if ori=0 then pSpeed=300    % faster for landscape
!-------------------------------
! Bucket bitmap ob1

gr.bitmap.load b1,"bucket.png"
gr.bitmap.size b1,kW,kH
kY = base-pH-kH             % bucket Y pos
gr.sprite.draw ob1,gWID/2-kW/2,kY,b1
collect = kY+kH*0.25        % collect drop pos
!-------------------------------
! Grass bitmap ob2

gr.bitmap.load temp,"grass.png"
gr.bitmap.size temp,,gH         % get grass height
gr.bitmap.scale b2,temp,gWID,gH % stretch horizontally
gr.bitmap.delete temp
gr.bitmap.draw ob2,b2,0,base-3  % grass below player
!-------------------------------
! Button sprites os10,os11

gr.bitmap.load b10,"b-left.png" % left button
gr.bitmap.load b11,"b-right.png"    % right button
gr.bitmap.size b10,bW,bH        % width and height
gr.anim.load aDown~             % one frame animation
,"b-pressed.png",bW,bH,1        % pressed button
                                
bx1=gWID/4-bW/2                 % button x positions
bx2=gWID*3/4-bW/2
by=base+gH*1.5                  % buttons below grass
gr.sprite.draw os10,bx1,by,b10
gr.sprite.draw os11,bx2,by,b11
gr.modify os10,"aloop",0        % don't loop buttons
gr.modify os11,"aloop",0        % when pressed
!-------------------------------
! Raindrop bitmaps drops[]

gr.bitmap.load bmp,"drop.png"
gr.bitmap.size bmp,,dH          % get raindrop height
dim spawned[20]                 % drops spawned ?
dim drops[20]                   % object array

for i=1 to 20
    x=(bW/2) + RND()*(gWID-bW)      % random x pos
    gr.bitmap.draw iDrop,bmp,x,-dH
    gr.modify iDrop,"dir",90        % direction is down
    drops[i] = iDrop
next
nxtdrop=1                           % next drop to fall
dtime=2000                          % time between drops
dSpeed=100                          % drop speed
if ori=0 then dSpeed=80             % slower for landscape
!-------------------------------
! Score

gr.text.size 36 : gr.text.align 2
gr.color "BLACK"
gr.text.draw t1,half,by+36,"0"      % score next to buttons
!-------------------------------
! Move the player & bucket as a group

gr.group pb, os0,ob1                % player+bucket group
!-------------------------------
touched=0
state=0                             % standing,left,right
score=0
nxt=time()+dtime                    % when to spawn next
gosub spawn                         % spawn first drop

GR.MOTION.MARK                      % first motion from now
while 1             % the game loop
    gr.render						% display objects
	gr.touch tch,tx,ty,1            % get touch (with unscale)

    if tch then                     % move player
        if state<>1 & gr_contains(os10,tx,ty)   % left button contains touch ?
            gr.modify os10,"anim",aDown %  press button (one frame)

            gr.modify os0,"anim",a1~    % player left animation
                         ,"aframe",0    % reset frame
            gr.modify pb,"dir",180~     % player & bucket go left
                        ,"speed",pSpeed % go player & bucket
            state=1
        else if state<>2 & gr_contains(os11,tx,ty)  % right button contains touch ?
            gr.modify os11,"anim",aDown %  press button (one frame)

            gr.modify os0,"anim",a2~    % player right animation
                        ,"aframe",0     % reset frame
            gr.modify pb,"dir",0~       % player & bucket go right
                        ,"speed",pSpeed % go player & bucket
            state=2
        endif
    else % no touch					% stop player
        if state<>0
            gr.modify os0,"anim",0  % restore standing sprite
            gr.modify pb,"speed",0  % stop player & bucket
            gr.modify os10,"anim",0 % restore buttons
            gr.modify os11,"anim",0
            state=0
        endif
    endif %_if touched

    GR.MOTION           % move objects
                        % check for collisions
    gr.get.value os0,"x",x          % get player position
    if x<0 then 
        gr.modify os0,"x",0         % left wall
        gr.modify ob1,"x",pw/2-kw/2 % left wall
    endif
    if x>gwid-pw then 
        gr.modify os0,"x",gWID-pw   % right wall
        gr.modify ob1,"x",gWID-pw/2-kw/2% right wall
    endif
    for i=1 to 20
        if spawned[i]
            gr.get.value drops[i],"y",y % if drop hits floor
            if y+dH>base then w_r.break % game over

            hit = gr_collision (drops[i],ob1) & (y+dH > collect)
            if hit                  % if drop touches bucket
                spawned[i] = 0      % un-spawn it
                gr.modify drops[i],"speed",0~   % stop it
                                  ,"y",-dH      % move it offscreen
                score++                         % well done
                gr.modify t1,"text",int$(score) % score
            endif %_hit
        endif %_spawned
    next
                        % spawn another drop
    if time() > nxt
        gosub spawn
        nxt = time()+dtime
    endif
repeat %_main loop

gr.close
if ori=0 then tSize=gHGT/20 else tSize=gWID/20
.cs "dividers off, textsize="+int$(tSize)
cls : console.screen w,h
print.at h/4+1,w/2-4; "Game Over":?
end
!-------------------------
spawn:                          % spawn a drop
    if spawned[nxtdrop] then return % reached max
    spawned[nxtdrop] = 1       % dropping
    gr.modify drops[nxtdrop],"speed",dSpeed % go
    nxtdrop++
    if nxtdrop >20 then nxtdrop=1
return
!-------------------------
end
